package se.phlogiston.grizzly;

import com.sun.grizzly.*;
import com.sun.grizzly.filter.ReadFilter;
import com.sun.grizzly.filter.EchoFilter;
import com.sun.grizzly.filter.LogFilter;

/**
 * Simple EchoServer using grizzly, copied from somewhere on grizzly.dev.java.net
  */
public class Server {

  final ProtocolFilter read = new ReadFilter();
  final ProtocolFilter echo = new EchoFilter();
  final TCPSelectorHandler tcp_handler = new TCPSelectorHandler();
  final Controller controller = new Controller();

  public static void main(String[] args) {
    int port = 0;

    for (int i = 0; i < args.length;i++) {
      if (args[i].equals("-p")) {
        if (args[++i] != null) {
          port = Integer.parseInt(args[i]);
          System.out.println("Port set to "+port);
        } else {
          System.out.println("No port set!");
        }
      }
    }
    Server s = new Server();
    s.execute(port);
  }

  void execute(int pPort) {
    if (pPort != 0) {
      tcp_handler.setPort(pPort);
    } else {
      System.out.println("No port given!");
      System.exit(1);
    }
    controller.setSelectorHandler(tcp_handler);
    controller.setProtocolChainInstanceHandler(
            new DefaultProtocolChainInstanceHandler() {
              public ProtocolChain poll() {
                ProtocolChain protocol_chain = protocolChains.poll();
                if (protocol_chain == null) {
                  protocol_chain = new DefaultProtocolChain();
                  protocol_chain.addFilter(read);
                  //protocol_chain.addFilter(log);
                  protocol_chain.addFilter(echo);
                  offer(protocol_chain);
                }
                return protocol_chain;
              }
            }); // end overridden method


    try {
      controller.start();
    } catch(Exception e) {
      System.out.println("Exception in controller...");
    }
  }
}
